/* .COPYRIGHT:                                                                */
/*    Licensed Material - Program-Property of IBM                             */
/*    (C) Copyright IBM Corp. 2001 - All Rights Reserved                      */
/*                                                                            */
/* .DISCLAIMER:                                                               */
/*   The following [enclosed] code is sample code created by IBM              */
/*   Corporation.  This sample code is provided to you solely for use in      */
/*   connection with your internal evaluation of the Service Management       */
/*   Framework(TM) Toolkit (Program) during the term of Early Release license */
/*   for the Program, and for no other purpose.  The code is provided 'AS IS',*/
/*   without warranty of any kind.  IBM shall not be liable for any damages   */
/*   arising out of your use of the sample code, even if they have been       */
/*   advised of the possibility of such damages.                              */

package com.ibm.osg.sample.handlesecurity;

import org.osgi.service.http.*;
import org.osgi.util.tracker.*;
import org.osgi.framework.*;
import org.osgi.service.useradmin.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.net.*;
import java.util.*;
import java.io.*;
import java.security.*;

/**
 * This is the HandleSecurity HttpContext class. This class implements HttpContext.
 * It is used as a "plugin" to the IBM Http Service implementation.  It defines the functionality of the
 * default HttpContext's handleSecurity method.
 *
 * This handleSecurity method implements HTTP Basic Authentication.  It uses the UserAdmin
 * service to validate usernames and passwords.
 *
 * The getResource and getMimeType methods in this class are ignored.
 */
public class HandleSecurity implements HttpContext
{
    /* UserAdmin interface name */
    static protected final String userAdminClazz = "org.osgi.service.useradmin.UserAdmin";

    /* ServiceTracker used to track the UserAdmin service */
    protected ServiceTracker userAdminTracker;

    /**
     * Constructor.
     *
     * @param context The BundleContext for this bundle.
     */
    public HandleSecurity(BundleContext context)
    {
        userAdminTracker = new ServiceTracker(context, userAdminClazz, null);

        /* Open ServiceTracker.  This will cause the userAdminTracker to start
         * tracking the UserAdmin service.
         */
        userAdminTracker.open();
    }

    /**
     * This handleSecurity method is called by the IBM Http Service implementation
     * when a servlet or resource is requested
     * that was registered with a default HttpContext.
     *
     * @param request The HTTP request object
     * @param response The HTTP response object
     */
    public boolean handleSecurity(HttpServletRequest request, HttpServletResponse response)
    {
        String auth = request.getHeader("Authorization");

        if (auth != null)
        {
            StringTokenizer tok = new StringTokenizer(auth);
            String authscheme = tok.nextToken();

            /* Check to see if we are using basic authentication.
             * This sample does Basic Authentication.
             */
            if (authscheme.equals("Basic"))
            {
                /* Get the username and password from the http request headers. */
                String base64credentials = tok.nextToken();

                /* Basic Authentication uses Base64 encoding.  Use BASE64 class to
                 * decode username and password.
                 */
                String credentials = BASE64.decode(base64credentials);
                int colon = credentials.indexOf(':');
                String userid = credentials.substring(0,colon);
                String password = credentials.substring(colon+1);

                Authorization authorization = validate(userid, password);

                if (authorization != null)
                {
                    request.setAttribute(HttpContext.REMOTE_USER, userid);
                    request.setAttribute(HttpContext.AUTHENTICATION_TYPE, authscheme);
                    request.setAttribute(HttpContext.AUTHORIZATION, authorization);

                    return(true);
                }
            }
        }

        response.setHeader("WWW-Authenticate", "Basic realm=\"SMF Handle Security Sample\"");

        try
        {
            response.sendError(response.SC_UNAUTHORIZED);
        }
        catch (IOException e)
        {
        }

        return(false);
    }

    /* This method is never called. */
    public URL getResource(String resource)
    {
        return(null);
    }

    /* This method is never called. */
    public String getMimeType(String resource)
    {
        return(null);
    }

    /**
     * The validate method checks a username and password against
     * data that is in the UserAdmin service.
     *
     * @param username The name of the user.
     * @param password The password of the user.
     * @return The UserAdmin authorization object for the user if the user
     * exists and has the correct password credential, null otherwise.
     */
    protected Authorization validate (final String username, final String password)
    {
        if (username == null || password == null)
        {
            return null;
        }

        /* Use the ServiceTracker (userAdminTracker) to get the UserAdmin
         * service.
         */
        final UserAdmin useradmin = (UserAdmin)userAdminTracker.getService();

        if (useradmin == null)
        {
            return null;
        }

        Authorization authorization = (Authorization) AccessController.doPrivileged(new PrivilegedAction()
        {
            public Object run()
            {
                Role role = useradmin.getRole(username);
                if (role instanceof User)
                {
                    User user = (User) role;
                    if (user.hasCredential("password",password))
                    {
                        return useradmin.getAuthorization(user);
                    }
                }

                return null;
            }
        });

        return authorization;
    }
}