/* .COPYRIGHT:                                                                */
/*    Licensed Material - Program-Property of IBM                             */
/*    (C) Copyright IBM Corp. 2001 - All Rights Reserved                      */
/*                                                                            */
/* .DISCLAIMER:                                                               */
/*   The following [enclosed] code is sample code created by IBM              */
/*   Corporation.  This sample code is provided to you solely for use in      */
/*   connection with your internal evaluation of the Service Management       */
/*   Framework(TM) Toolkit (Program) during the term of Early Release license */
/*   for the Program, and for no other purpose.  The code is provided 'AS IS',*/
/*   without warranty of any kind.  IBM shall not be liable for any damages   */
/*   arising out of your use of the sample code, even if they have been       */
/*   advised of the possibility of such damages.                              */

package com.ibm.osg.sample.simplepermissionpolicy;

import java.io.*;
import java.util.*;
import java.net.*;
import org.osgi.framework.*;
import org.osgi.service.permissionadmin.*;
import org.osgi.util.tracker.*;

/**
 * This class is the main class for the SimplePermissionPolicy sample.
 * It is the
 * BundleActivator and is responsible for starting and stopping the
 * bundle. It also manages the use of the HttpService and registers
 * the Pizza servlet.
 */
public class SimplePermissionPolicy implements BundleActivator, SynchronousBundleListener
{
    protected static final String permissionResource = "/META-INF/permission.info";
    protected static final String defaultPermissionResource = "/META-INF/permission.defaults";

    /** BundleContext for this bundle */
    protected BundleContext context;

    /** Log Service wrapper object */
    protected LogTracker log;

    /** PermissionAdmin Service tracker object */
    protected ServiceTracker tracker;

    /**
     * Must have a public default constructor!
     * It doesn't have to do anything but the BundleActivator
     * class must be able to be instantiated by Class.newInstance().
     *
     * This same instance of this object that is used to start
     * the bundle will be used to stop the bundle.
     */
    public SimplePermissionPolicy()
    {
    }

    /**
     * Part of the BundleActivator interface. This method starts
     * the bundle and must quickly return. If the bundle requires
     * its own threads. they can be created here. If the bundle
     * registers any Services, they can be registered here.
     *
     * @param   context BundleContext object for this bundle
     */
    public void start(BundleContext context) throws Exception
    {
        this.context = context;

        log = new LogTracker(context, System.err);

        tracker = new ServiceTracker(context, PermissionAdmin.class.getName(), null);
        tracker.open();

        context.addBundleListener(this);

        initialize();
    }

    /**
     * Part of the BundleActivator interface. This method stops
     * the bundle and must quickly return. If the bundle registered any
     * Services they should be unregistered here.
     *
     * @param   context BundleContext object for this bundle
     */
    public void stop(BundleContext context) throws Exception
    {
        tracker.close();
        log.close();
    }

    public void initialize()
    {
        PermissionAdmin permissionAdmin = (PermissionAdmin)tracker.getService();

        if (permissionAdmin == null)   /* no permission admin service */
        {
            return;
        }

        /* Set my permissions if they are not already set.
         * This only happens once, the first time I am started
         * after being installed.
         */

        Bundle self = context.getBundle();
        String location = self.getLocation();

        PermissionInfo[] info = permissionAdmin.getPermissions(location);

        if (info == null)
        {
            setPermissions(self);
        }

        /* Set the default permissions if they are not already set.
         * This only happens once, the first time I am started
         * after being installed.
         */

        info = permissionAdmin.getDefaultPermissions();

        if (info == null)
        {
            try
            {
                info = getPermissions(getClass().getResourceAsStream(defaultPermissionResource));
            }
            catch (IOException e)
            {
                log.log(log.LOG_ERROR, "Unable to read default permission info", e);
            }

            if (info != null)
            {
                permissionAdmin.setDefaultPermissions(info);

                log.log(log.LOG_INFO, "Set default permissions");
            }
        }

        /* Since some bundles may have been installed, updated
         * or uninstalled while I was not running. I need to update
         * PermissionAdmin based up current set of installed bundles.
         */

        /* Get the list of locations in the PermissionAdmin database */
        String[] locations = permissionAdmin.getLocations();
        int size = (locations == null) ? 0 : locations.length;

        /* Copy them into this vector which we will shrink as we process installed bundles */
        Vector uninstalledBundles = new Vector(size);
        for (int i = 0; i < size; i++)
        {
            uninstalledBundles.addElement(locations[i]);
        }

        /* update the permissions data for all installed bundles */
        Bundle[] bundles = context.getBundles();
        size = bundles.length;

        for (int i = 0; i < size; i++)
        {
            Bundle bundle = bundles[i];

            /* remove from uninstalled list */
            uninstalledBundles.removeElement(bundle.getLocation());

            if ((bundle.getBundleId() == 0) || bundle.equals(self))
            {
                continue;
            }

            /* update the permissions for the bundle */
            setPermissions(bundle);
        }

        /* remove the permissions data for all bundles which are no longer installed */
        size = uninstalledBundles.size();
        for (int i = 0; i < size; i++)
        {
            location = (String) uninstalledBundles.elementAt(i);

            /* removed the permissions for the no longer installed bundle */
            clearPermissions(location);
        }
    }

    public void setPermissions(Bundle bundle)
    {
        PermissionAdmin permissionAdmin = (PermissionAdmin)tracker.getService();

        if (permissionAdmin == null)   /* no permission admin service */
        {
            return;
        }

        PermissionInfo[] info = getPermissions(bundle);

        permissionAdmin.setPermissions(bundle.getLocation(), info);

        log.log(log.LOG_INFO, "Set permissions for bundle id: "+bundle.getBundleId());
    }

    public PermissionInfo[] getPermissions(Bundle bundle)
    {
        URL url = bundle.getResource( permissionResource);

        PermissionInfo[] info = null;

        if (url != null)
        {
            try
            {
                InputStream in = url.openStream();

                info = getPermissions(in);
            }
            catch (IOException e)
            {
                log.log(log.LOG_ERROR, "Unable to read permission info for bundle id  "+bundle.getBundleId(), e);
            }
        }

        return info;
    }

    public PermissionInfo[] getPermissions(InputStream in) throws IOException
    {
        PermissionInfo[] info = null;

        if (in != null)
        {
            Vector permissions = new Vector();

            try
            {
                BufferedReader reader = new BufferedReader(new InputStreamReader(in));

                while (true)
                {
                    String line = reader.readLine();

                    if (line == null)   /* EOF */
                    {
                        break;
                    }

                    line = line.trim();

                    if ((line.length() == 0) || line.startsWith("#") || line.startsWith("//"))  /* comments */
                    {
                        continue;
                    }

                    try
                    {
                        permissions.addElement(new PermissionInfo(line));
                    }
                    catch (IllegalArgumentException iae)
                    {
                        /* incorrectly encoded permission */

                        log.log(log.LOG_WARNING, "Permission incorrectly encoded: "+line, iae);
                    }
                }
            }
            finally
            {
                in.close();
            }

            int size = permissions.size();

            if (size > 0)
            {
                info = new PermissionInfo[size];

                permissions.copyInto(info);
            }
        }

        return info;
    }

    public void clearPermissions(String location)
    {
        PermissionAdmin permissionAdmin = (PermissionAdmin)tracker.getService();

        if (permissionAdmin == null)   /* no permission admin service */
        {
            return;
        }

        permissionAdmin.setPermissions(location, null);

        log.log(log.LOG_INFO, "Cleared permissions for location: "+location);
    }

    public void bundleChanged(BundleEvent event)
    {
        Bundle bundle = event.getBundle();

        if (bundle.getBundleId() == 0)  /* ignore the system bundle */
        {
            return;
        }

        int type = event.getType();

        switch (type)
        {
            case BundleEvent.INSTALLED:
            case BundleEvent.UPDATED:
                setPermissions(bundle);
                break;

            case BundleEvent.UNINSTALLED:
                clearPermissions(bundle.getLocation());
                break;
        }
    }
}
