/* .COPYRIGHT:                                                                */
/*    Licensed Material - Program-Property of IBM                             */
/*    (C) Copyright IBM Corp. 2002 - All Rights Reserved                      */
/*                                                                            */
/* .DISCLAIMER:                                                               */
/*   The following [enclosed] code is sample code created by IBM              */
/*   Corporation.  This sample code is provided to you solely for use in      */
/*   connection with your internal evaluation of the Service Management       */
/*   Framework(TM) Toolkit (Program) during the term of Early Release license */
/*   for the Program, and for no other purpose.  The code is provided 'AS IS',*/
/*   without warranty of any kind.  IBM shall not be liable for any damages   */
/*   arising out of your use of the sample code, even if they have been       */
/*   advised of the possibility of such damages.                              */
package com.ibm.pvc.example.calendar;

import java.net.URL;
import java.security.AccessController;
import java.security.PrivilegedAction;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceEvent;
import org.osgi.framework.ServiceListener;
import org.osgi.framework.ServiceReference;
import org.osgi.service.http.HttpContext;
import org.osgi.service.http.HttpService;
import org.osgi.service.http.NamespaceException;

import com.ibm.pvc.example.calendar.model.command.CreateDbCommand;

/**
 * This is the CalendarBundle class.
 * 
 * This class is the calendar example's Bundle Activator.  This class 
 * is responsible for starting and stopping the bundle. It also 
 * manages the use of the HttpService and registers the Calendar
 * servlet
 * 
 * The Calendar application stores the calendar events in a DB2e database.
 * This database is created by this class the first time this class is 
 * called on to start this bundle, 
 */
public class CalendarBundle implements BundleActivator, ServiceListener {
	/** BundleContext for this bundle */
	protected BundleContext context;

	/** HttpService interface class name */
	protected final String clazz = HttpService.class.getName();

	/** HttpService service object */
	protected HttpService service;

	/** HttpService service reference */
	protected ServiceReference reference;

	/** HttpContext for HTTP registrations */
	protected HttpContext httpContext;

	/** Servlet images URI suffix */
	protected static final String IMAGES_RESOURCE_URI = "/images";

	/** Servlet images URI suffix */
	protected static final String HELP_RESOURCE_URI = "/help_files";

	/**
	 * Must have a public default constructor!
	 * It doesn't have to do anything but the BundleActivator
	 * class must be able to be instantiated by Class.newInstance().
	 *in
	 * This same instance of this object that is used to start
	 * the bundle will be used to stop the bundle.
	 */
	public CalendarBundle() {
		httpContext = new HttpContext() {
			public boolean handleSecurity(
				HttpServletRequest request,
				HttpServletResponse response) {
				return (true);
			}

            public URL getResource(final String name)
            {			
               return (URL) AccessController.doPrivileged(new PrivilegedAction()
                {
                    public Object run()
                    {
                        String resource = name;

                        if (resource.charAt(0) != '/')
                        {
                            resource = "/".concat(resource);
                        }

                        return getClass().getResource(resource);
                    }
                });		
            }	

			public String getMimeType(String name) {
				return (null);
			}
		};
	}
	/**
	 * Get the service object.
	 *
	 * @param reference ServiceReference to a service.
	 */
	protected synchronized void getService(ServiceReference reference) {
		if (service == null) {
			this.reference = reference;

			if (reference != null) {
				service = (HttpService) context.getService(reference);

				if (service == null) {
					reference = null;
				}

				registerServlet();
			}
		}
	}
	/**
	 * Register the Servlet.
	 *
	 */
	protected void registerServlet() {
		if (service != null) /* if service available */ {
			try {
				service.registerServlet(
					CalendarServlet.SERVLET_URI,
					new CalendarServlet(),
					null,
					httpContext);
				service.registerResources(
					CalendarServlet.IMAGES_URI,
					CalendarBundle.IMAGES_RESOURCE_URI,
					httpContext);

				service.registerResources(
					CalendarServlet.HELP_URI,
					CalendarBundle.HELP_RESOURCE_URI,
					httpContext);
			}
			catch (ServletException e) {
				Throwable n = e.getRootCause();
				if (n == null) {
					n = e;
				}
				ungetService();
			}
			catch (NamespaceException e) {
				Throwable n = e.getException();
				if (n == null) {
					n = e;
				}
				ungetService();
			}
		}
	}
	/**
	 * ServiceListener for this object.
	 *
	 * @param event ServiceEvent.
	 */
	public void serviceChanged(ServiceEvent event) {
		ServiceReference reference = event.getServiceReference();

		switch (event.getType()) {
			case ServiceEvent.REGISTERED :
				{
					if (this.reference == null) {
						getService(reference);
					}

					break;
				}

			case ServiceEvent.UNREGISTERING :
				{
					if (reference.equals(this.reference)) {
						ungetService();
					}

					break;
				}
		}
	}
	/**
	 * Part of the BundleActivator interface. This method starts
	 * the bundle and must quickly return. If the bundle requires
	 * its own threads. they can be created here. If the bundle
	 * registers any Services, they can be registered here.
	 *
	 * @param   context BundleContext object for this bundle
	 */
	public void start(BundleContext context) throws Exception {
		try {
			CreateDbCommand
				createDbCommand = new CreateDbCommand();
				
			createDbCommand.setDoNothingIfDbExists(true);
		
			createDbCommand.execute();
		}
		catch (Exception ex) {
			// For this example app, this error will only be printed to System.out
			System.out.println("createDbCommand failed:" + ex.toString());
		}			
		this.context = context;

		context.addServiceListener(this, "(objectClass=" + clazz + ")");

		getService(context.getServiceReference(clazz));
	}
	/**
	 * Part of the BundleActivator interface. This method stops
	 * the bundle and must quickly return. If the bundle registered any
	 * Services they should be unregistered here.
	 *
	 * @param   context BundleContext object for this bundle
	 */
	public void stop(BundleContext context) throws Exception {
		context.removeServiceListener(this);

		ungetService();

		this.context = null;
	}
	/**
	 * Release the service object.
	 *
	 */
	protected synchronized void ungetService() {
		if (service != null) {
			unregisterServlet();

			service = null;

			context.ungetService(reference);
		}

		reference = null;
	}
	/**
	 * Unregister the Servlet.
	 *
	 */
	protected void unregisterServlet() {
		if (service != null) {
			try {
				service.unregister(CalendarServlet.SERVLET_URI);
			}
			catch (IllegalArgumentException e) {
			}

			try {
				service.unregister(CalendarServlet.IMAGES_URI);
			}
			catch (IllegalArgumentException e) {
			}
		}
	}
}